from dataclasses import dataclass, field
from typing import List

# This is an example of VIIRS config - it can also be used as a way to check that we have the right dims/coords/variables in the 

@dataclass
class VIIRSData():
    # This should hopefully allow us to make some changes if the NOAA data is slightly different to SNPP
    # We could add metadata values if we want to include the attributes of the variables
    groups: bool = False
    name_changes: bool = True
    dims: List = field(default_factory=lambda:
                            [
                                'Idx_Atrack',
                                'Idx_Xtrack',
                                'Land_Bands',
                                'Ocean_Bands', 
                                'Reflectance_Bands'
                            ]
                           )
        
    coords: List = field(default_factory=lambda:
                              [
                                  'Idx_Atrack', 
                                  'Idx_Xtrack', 
                                  'Land_Bands', 
                                  'Ocean_Bands', 
                                  'Reflectance_Bands', 
                                  'Latitude', 
                                  'Longitude'
                              ]
                             )
    variables: List = field(default_factory=lambda:
                                 [
                                    'Aerosol_Optical_Thickness_550_Expected_Uncertainty_Land',
                                    'Aerosol_Optical_Thickness_550_Expected_Uncertainty_Ocean',
                                    'Aerosol_Optical_Thickness_550_Land',
                                    'Aerosol_Optical_Thickness_550_Land_Best_Estimate',
                                    'Aerosol_Optical_Thickness_550_Land_Ocean',
                                    'Aerosol_Optical_Thickness_550_Land_Ocean_Best_Estimate',
                                    'Aerosol_Optical_Thickness_550_Ocean',
                                    'Aerosol_Optical_Thickness_550_Ocean_Best_Estimate',
                                    'Aerosol_Optical_Thickness_550_STDV_Land',
                                    'Aerosol_Optical_Thickness_550_STDV_Ocean',
                                    'Aerosol_Optical_Thickness_QA_Flag_Land',
                                    'Aerosol_Optical_Thickness_QA_Flag_Ocean',
                                    'Aerosol_Type_Land',
                                    'Aerosol_Type_Land_Ocean',
                                    'Aerosol_Type_Ocean',
                                    'Algorithm_Flag_Land',
                                    'Algorithm_Flag_Ocean',
                                    'Angstrom_Exponent_Land',
                                    'Angstrom_Exponent_Land_Best_Estimate',
                                    'Angstrom_Exponent_Land_Ocean',
                                    'Angstrom_Exponent_Land_Ocean_Best_Estimate',
                                    'Angstrom_Exponent_Ocean',
                                    'Angstrom_Exponent_Ocean_Best_Estimate',
                                    'Cell_Average_Elevation_Land',
                                    'Cell_Average_Elevation_Ocean',
                                    'Fine_Mode_Fraction_550_Ocean',
                                    'Fine_Mode_Fraction_550_Ocean_Best_Estimate',
                                    'Number_Of_Pixels_Used_Land',
                                    'Number_Of_Pixels_Used_Ocean',
                                    'Number_Valid_Pixels',
                                    'Ocean_Sum_Squares',
                                    'Precipitable_Water',
                                    'Relative_Azimuth_Angle',
                                    'Scan_Start_Time',
                                    'Scattering_Angle',
                                    'Solar_Zenith_Angle',
                                    'Spectral_Aerosol_Optical_Thickness_Land',
                                    'Spectral_Aerosol_Optical_Thickness_Ocean',
                                    'Spectral_Single_Scattering_Albedo_Land',
                                    'Spectral_Surface_Reflectance',
                                    'Spectral_TOA_Reflectance_Land',
                                    'Spectral_TOA_Reflectance_Ocean',
                                    'TOA_NDVI',
                                    'Total_Column_Ozone',
                                    'Unsuitable_Pixel_Fraction_Land_Ocean',
                                    'Viewing_Zenith_Angle',
                                    'Wind_Direction',
                                    'Wind_Speed'
                                ]
                            )
        