
import time
import numpy as np
import pandas as pd
import xarray as xr

from functools import wraps
from glob import glob
# Suggestion from Guillaume which I like the look of
import logging
from typing import List
from datetime import datetime



#Timing decorator
def timeit(func):
    @wraps(func)
    def timeit_wrapper(*args, **kwargs):
        start_time = time.perf_counter()
        result = func(*args, **kwargs)
        end_time = time.perf_counter()
        total_time = end_time - start_time
        print(f'Function {func.__name__}{args} {kwargs} Took {total_time:.4f} seconds')
        return result
    return timeit_wrapper


# Function from Guillaume's script could be useful
# Only supported in python3.10
#def get_file_list(patterns: str | List[str]) -> List[str]:
def get_file_list(patterns) -> List[str]:
    """
    :param patterns: one or several glob patterns (or exact file names) pointing to the location of files on disk.
    :return: the matching list of files
    """

    # 1st make sure we have a list of patterns
    if isinstance(patterns, str):
        patterns = [patterns]

    # Then decode them one by one
    files = []
    for pattern in patterns:
        logging.debug(f"getting file list matching {pattern}")
        files.extend(glob(pattern))

    return files

def exception_factory(exception, message):
    return exception(message)




def time_converter(date:str) -> pd.Timestamp:
    """
    :param date: date that you wish convert to standardised format
    """
    # Remove any special characters
    for ch in ['\\','`','*','_','{','}','[',']','(',')','>','#','+','-','.','!','$','\'', ':', ' ']:
        if ch in date:
            date = date.replace(ch,"")
            logger.info(f"Special character {ch} was replaced in the date")
    
    if len(date) == 14: # Probable str format %Y%m%d%H%M%S
        date = pd.to_datetime(date, format="%Y%m%d%H%M%S")
    elif len(date) == 12: # Probable str format %Y%m%d%H%M
        date = pd.to_datetime(date, format="%Y%m%d%H%M")
    elif len(date) == 10: # Probable str format %Y%m%d%H
        date = pd.to_datetime(date, format="%Y%m%d%H")
    elif len(date) == 8: # Probable str format %Y%m%d
        date = pd.to_datetime(date, format="%Y%m%d%H")
    elif len(date) == 7: # Probable str format %Y%J
        date =pd.to_datetime(date, format="%Y%j")
    else:
        raise exception_factory(ValueError, "Invalid date format")

    return date



# Time Domain Selection function based off of xarray Will this work with dask too, a question for later
def time_domain_selection(
        ds: xr.Dataset, 
        time_column, 
        start_date, 
        end_date, 
        closed = None) -> xr.Dataset:
    """
    :param ds: one or several xarray datasets to cut down to correct size.
    :param start_date: the start date that we want to start with for our time domain selection
    :param end_date: the end date that we want to end with for our time domain seleciton
    :param time_column: the column for the time variables
    :return: a cut down dataset
    """

    # But the format of the time column may change aaaaaaaghh

    # For these if it is a string we need to know what the format is (maybe pandas figures that out)
    if isinstance(start_date, str):
        start_date = pd.to_datetime(start_date)

    if isinstance(end_date, str):
        end_date = pd.to_datetime(end_date)

    
    # For the time column we can either centralise it in the seperate Instrument objects or pass it here,
    # I will start by passing it here

    def is_between_times(array):
        """
        :param closed: like the pandas old param on the pandas date_range method (None, left, right), default:None
        """
        if closed == "left":
            return (array > start_date) & (array <= end_date)
        elif closed == "right":
            return (array >= start_date) & (array < end_date)
        else:
            return (array >= start_date) & (array <= end_date)

    # I need to make sure that the time column is a datetime column # Need a time orig
    print(ds[time_column].values)
    ds["time"] = datetime.strptime("199301010000", "%Y%m%d%H%M") + df[time_column]
    time_sel_ds = ds.sel(time=is_between_times(ds["time"]))

    return time_sel_ds
    